from . import consts 

class buildPacket:
    def __init__(self, channel: consts.Channels, raw: bytes):
        self.channel = channel
        self.raw = raw

    def assembly(self) -> bytes:
        packet = bytes()
        for i in consts.FCP:
            packet += int(i).to_bytes(2, byteorder = "big")
        packet += int(self.channel).to_bytes(2, byteorder = "big")
        for r in self.raw:
            packet += int(r).to_bytes(2, byteorder = "big")
        return packet




class stripPacket:
    def __init__(self):
        pass

    def disassembly(self, packet : bytes) -> None:
        fulltag = []
        for t in packet[0:6]:
            if packet.index(t) % 2 != 0:
                fulltag.append(chr(t))
        if "fcp" != "".join(fulltag):
            raise Exception("Packet not valid: FCP Tag not found.")
        self.ch = int.from_bytes(packet[6:8], byteorder = "big")
        fullraw = []
        for i in packet[8:]:
            if packet.index(i) % 2 != 0:
                fullraw.append(i)
        self.rw = fullraw
    
    @property
    def channel(self) -> int:
        """
        Get the channel.
        """
        return self.ch

    @property
    def raw(self) -> list[int]:
        """
        Get the raw.
        """
        return self.rw


from . import assemblr, disassemblr
from .consts import *
from scapy.all import *
from scapy.layers.inet import Ether
from cryptography.utils import CryptographyDeprecationWarning
import warnings
warnings.filterwarnings(action = "ignore", category = CryptographyDeprecationWarning)

class FCPSocket:
    def __init__(self, channel: Channels):
        self.ch = channel
        self.sock = conf.L2socket()

    def send(self, raw: bytes):
        packet = assemblr.buildPacket(self.ch, raw).assembly()
        sendp(Ether(dst = "ff:ff:ff:ff:ff:ff")/(packet))

    def recv(self):
        while True:
            data = self.sock.recv()
            if data is not None:
                try: 
                    data = data[Raw].load 
                    fulltag = []
                    for t in data[0:6]:
                        if data.index(t) % 2 != 0:
                            fulltag.append(chr(t))
                    if "fcp" != "".join(fulltag):
                        continue
                except:
                    continue 
                dis = disassemblr.stripPacket()
                dis.disassembly(data)
                if dis.channel == self.ch:
                    return bytes(dis.raw)
            


        


from .FCPSocket import *
from .consts import *
import warnings
from cryptography.utils import CryptographyDeprecationWarning
warnings.filterwarnings(action = "ignore", category = CryptographyDeprecationWarning)

AUTHOR = "Andrea Vaccaro"


from setuptools import setup
import os, codecs

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = "0.0.3"
DESCRIPTION = 'FCP is a protocol used to route on "channels" and send or receive data.'

setup(
    name = "protofcp",
    version = VERSION,
    url = "https://github.com/ANDRVV/FCP",
    author = "Andrea Vaccaro",
    description = DESCRIPTION,
    long_description = long_description,
    long_description_content_type = "text/markdown",
    packages = ["protofcp"],
    license = "Apache 2.0",
    keywords = ["python", "socket", "protocol", "proto", "channel", "scapy", "packet"],
    install_requires = ["scapy"],
    classifiers = [
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows"]
    )



# FCP TAG

FCP =       [
            ord("f"),
            ord("c"), 
            ord("p")
            ]

# CHANNEL CONSTANTS

class Channels:
    CHANNEL_01 =      0x3e8
    CHANNEL_02 =      0x3e9
    CHANNEL_03 =      0x3ea
    CHANNEL_04 =      0x3eb
    CHANNEL_05 =      0x3ec
    CHANNEL_06 =      0x3ed
    CHANNEL_07 =      0x3ee
    CHANNEL_08 =      0x3ef
    CHANNEL_09 =      0x3f0
    CHANNEL_10 =      0x3f1